﻿using IRaCIS.Core.Application.Contracts;
using IRaCIS.Core.Application.Contracts.DTO;
using IRaCIS.Core.Application.Service.Reading.Dto;
using IRaCIS.Core.Domain.Share;
using IRaCIS.Core.Infrastructure;
using MassTransit.Initializers;
using Microsoft.AspNetCore.Mvc;

namespace IRaCIS.Core.Application.Image.QA
{
    [ApiExplorerSettings(GroupName = "Image")]
    public class QCListService(IRepository<VisitTask> _visitTaskRepository,
        IRepository<SubjectVisit> _subjectVisitRepository,
        IRepository<Trial> _trialRepository,
        IRepository<ClinicalDataTrialSet> _clinicalDataTrialSetRepository,
        IRepository<ClinicalDataTrialSet> _clinicalDataTrialSet,
        IRepository<TrialQCQuestionAnswer> _trialQCQuestionAnswerRepository,
        IRepository<TrialQCQuestion> _trialQCQuestionRepository,
        IRepository<DicomStudy> _dicomStudyRepository,
        IRepository<TrialUserRole> _trialUserRoleRepository,
        IRepository<DicomSeries> _dicomSeriesRepository,
        IRepository<NoneDicomStudy> _noneDicomStudyRepository,
        IRepository<QCChallenge> _qcChallengeRepository,
        IReadingImageTaskService _IReadingImageTaskService, IMapper _mapper, IUserInfo _userInfo, IStringLocalizer _localizer) : BaseService, IQCListService
    {






        #region CRC上传、质疑页面
        /// <summary>
        /// CRC 访视上传列表
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<IResponseOutput<PageOutput<QCCRCVisitViewModel>>> GetCRCVisitList(CRCVisitSearchDTO inQuery)
        {
            var svExpression = QCCommon.GetSubjectVisitFilter(inQuery.VisitPlanArray);

            var query = _subjectVisitRepository.Where(x => x.TrialId == inQuery.TrialId)
                       .Where(t => t.Subject.FinalSubjectVisitId != null ? t.VisitNum <= t.Subject.FinalSubjectVisit.VisitNum : true)
                       .WhereIf(inQuery.TrialSiteId != null, t => t.TrialSiteId == inQuery.TrialSiteId)
                       .WhereIf(inQuery.SubjectId != null, t => t.Subject.Id == inQuery.SubjectId)
                       .WhereIf(!string.IsNullOrEmpty(inQuery.SubjectInfo), t => t.Subject.Code.Contains(inQuery.SubjectInfo))

                       .WhereIf(inQuery.VisitPlanArray != null && inQuery.VisitPlanArray?.Length > 0, svExpression)
                       //.WhereIf(!string.IsNullOrEmpty(visitSearchDTO.VisitPlanInfo), visitSearchDTO.VisitPlanInfo.Contains('.') ? t => t.InPlan == false : t => t.VisitNum == decimal.Parse(visitSearchDTO.VisitPlanInfo))
                       .WhereIf(inQuery.AuditStateArray != null && inQuery.AuditStateArray?.Length > 0, t => inQuery.AuditStateArray!.Contains(t.AuditState))
                       .WhereIf(inQuery.SubmitState != null, t => t.SubmitState == inQuery.SubmitState)
                       .WhereIf(inQuery.ChallengeState != null, t => t.ChallengeState == inQuery.ChallengeState)
                       .WhereIf(inQuery.IsUrgent != null, t => t.IsUrgent == inQuery.IsUrgent)
                       .WhereIf(_userInfo.UserTypeEnumInt == (int)UserTypeEnum.ClinicalResearchCoordinator || _userInfo.UserTypeEnumInt == (int)UserTypeEnum.CRA, t => t.TrialSite.CRCUserList.Any(t => t.UserId == _userInfo.UserRoleId))
                       .ProjectTo<QCCRCVisitViewModel>(_mapper.ConfigurationProvider);


            var defalutSortArray = new string[] { nameof(QCCRCVisitViewModel.IsUrgent) + " desc", nameof(QCCRCVisitViewModel.SubjectCode), nameof(QCCRCVisitViewModel.VisitNum) };
            var pageList = await query.ToPagedListAsync(inQuery, defalutSortArray);

            var config = await _trialRepository.Where(t => t.Id == inQuery.TrialId).ProjectTo<TrialSubjectAndSVConfig>(_mapper.ConfigurationProvider).FirstOrDefaultAsync().IfNullThrowException();
            config.IsHaveSubjectClinicalData = await _clinicalDataTrialSet.AnyAsync(x => x.TrialId == inQuery.TrialId && x.IsConfirm && (x.ClinicalDataLevel == ClinicalLevel.Subject) && x.UploadRole == UploadRole.CRC);
            config.IsHaveVisitClinicalData = await _clinicalDataTrialSet.AnyAsync(x => x.TrialId == inQuery.TrialId && x.IsConfirm && (x.ClinicalDataLevel == ClinicalLevel.SubjectVisit) && x.UploadRole == UploadRole.CRC);
            return ResponseOutput.Ok(pageList, config);

        }

        /// <summary>
        /// 获取下一个crc的未关闭的质疑
        /// </summary>
        /// <param name="inDto"></param>
        /// <returns></returns>
        /// <exception cref="BusinessValidationFailedException"></exception>
        [HttpPost]
        public async Task<QCCRCChallengeViewModel> GetNextCRCChallenge(GetNextCRCChallengeInDto inDto)
        {
            var list = await GetCRCChallengeList(new ChallengeQuery()
            {
                TrialId = inDto.TrialId,
                PageIndex = 1,
                PageSize = 9999
            });

            var pageList = list.Data.CurrentPageData.ToList();


            var data = pageList.Where(x => x.Id == inDto.QCChallengeId || x.IsClosed == false).ToList();

            if (data.Count > 0)
            {

                var index = data.ToList().FindIndex(x => x.Id == inDto.QCChallengeId);

                var result = new QCCRCChallengeViewModel() { };

                if (index + 1 == data.Count()) // 最后一个
                {
                    throw new BusinessValidationFailedException(_localizer["QCList_CRCFinishChallenge"]);
                }
                else if (index == -1 || data.Count == 1) // 第一个或者只有一个
                {
                    if (data[0].Id == inDto.QCChallengeId)
                    {
                        throw new BusinessValidationFailedException(_localizer["QCList_CRCFinishChallenge"]);
                    }
                    result = data[0];
                }
                else
                {
                    result = data.Skip(index + 1).Take(1).First();

                }
                return result;

            }
            else
            {
                throw new BusinessValidationFailedException(_localizer["QCList_CRCFinishChallenge"]);
            }

        }



        /// <summary>
        /// CRC 质疑列表
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<IResponseOutput<PageOutput<QCCRCChallengeViewModel>>> GetCRCChallengeList(ChallengeQuery inQuery)
        {


            var svExpression = QCCommon.GetQCChallengeFilter(inQuery.VisitPlanArray);

            var query = _qcChallengeRepository.Where(x => x.TrialId == inQuery.TrialId)
                //.WhereIf(challengeQuery.ChallengeState != null, t => t.SubjectVisit.ChallengeState == challengeQuery.ChallengeState)
                .WhereIf(inQuery.ReuploadEnum != null, t => t.ReuploadEnum == inQuery.ReuploadEnum)
                .WhereIf(inQuery.IsClosed != null, t => t.IsClosed == inQuery.IsClosed)
                .WhereIf(inQuery.TrialSiteId != null, t => t.SubjectVisit.TrialSiteId == inQuery.TrialSiteId)
                .WhereIf(inQuery.SubjectId != null, t => t.SubjectVisit.SubjectId == inQuery.SubjectId)
                .WhereIf(inQuery.CreateUserId != null, t => t.CreateUserId == inQuery.CreateUserId)
                .WhereIf(inQuery.SubjectCode != null, t => t.SubjectVisit.Subject.Code.Contains(inQuery.SubjectCode!))
                .WhereIf(inQuery.VisitPlanArray != null && inQuery.VisitPlanArray?.Length > 0, svExpression)
                 //.WhereIf(!string.IsNullOrEmpty(challengeQuery.VisitPlanInfo), challengeQuery.VisitPlanInfo.Contains('.') ? t => t.SubjectVisit.VisitNum.ToString().Contains(".") : t => t.SubjectVisit.VisitNum == decimal.Parse(challengeQuery.VisitPlanInfo))
                 .WhereIf(inQuery.IsOverTime != null && inQuery.IsOverTime == true, t => t.IsClosed ? t.ClosedTime > t.DeadlineTime : DateTime.Now > t.DeadlineTime)
                .WhereIf(inQuery.IsOverTime != null && inQuery.IsOverTime == false, t => t.IsClosed ? t.ClosedTime < t.DeadlineTime : DateTime.Now < t.DeadlineTime)
                .WhereIf(inQuery.IsUrgent != null, t => t.SubjectVisit.IsUrgent == inQuery.IsUrgent)
                .WhereIf(_userInfo.UserTypeEnumInt == (int)UserTypeEnum.ClinicalResearchCoordinator || _userInfo.UserTypeEnumInt == (int)UserTypeEnum.CRA, t => t.SubjectVisit.TrialSite.CRCUserList.Any(t => t.UserId == _userInfo.UserRoleId))
              .ProjectTo<QCCRCChallengeViewModel>(_mapper.ConfigurationProvider);

            var pageList = await query.ToPagedListAsync(inQuery, new string[] { nameof(QCCRCChallengeViewModel.IsUrgent) + " desc", nameof(QCCRCChallengeViewModel.CreateTime) });

            var config = await _trialRepository.Where(t => t.Id == inQuery.TrialId).ProjectTo<TrialSubjectAndSVConfig>(_mapper.ConfigurationProvider).FirstOrDefaultAsync().IfNullThrowException();
            config.IsHaveSubjectClinicalData = await _clinicalDataTrialSet.AnyAsync(x => x.TrialId == inQuery.TrialId && x.IsConfirm && (x.ClinicalDataLevel == ClinicalLevel.Subject) && x.UploadRole == UploadRole.CRC);
            config.IsHaveVisitClinicalData = await _clinicalDataTrialSet.AnyAsync(x => x.TrialId == inQuery.TrialId && x.IsConfirm && (x.ClinicalDataLevel == ClinicalLevel.SubjectVisit) && x.UploadRole == UploadRole.CRC);
            return ResponseOutput.Ok(pageList, config);

        }

        #endregion


        #region QC 质疑 质控页面

        /// <summary>
        /// QC 质疑列表   分页
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<(PageOutput<QCCRCChallengeViewModel>, TrialSubjectAndSVConfig)> GetQCChallengeList(ChallengeQuery inQuery)
        {

            var svExpression = QCCommon.GetQCChallengeFilter(inQuery.VisitPlanArray);

            var query = _qcChallengeRepository.Where(x => x.TrialId == inQuery.TrialId)
                //.WhereIf(challengeQuery.ChallengeState != null, t => t.SubjectVisit.ChallengeState == challengeQuery.ChallengeState)
                .WhereIf(inQuery.ReuploadEnum != null, t => t.ReuploadEnum == inQuery.ReuploadEnum)
                .WhereIf(inQuery.IsClosed != null, t => t.IsClosed == inQuery.IsClosed)
                .WhereIf(inQuery.TrialSiteId != null, t => t.SubjectVisit.TrialSiteId == inQuery.TrialSiteId)
                .WhereIf(inQuery.SubjectId != null, t => t.SubjectVisit.SubjectId == inQuery.SubjectId)
                .WhereIf(inQuery.CreateUserId != null, t => t.CreateUserId == inQuery.CreateUserId)
                .WhereIf(!string.IsNullOrEmpty(inQuery.SubjectCode), t => t.SubjectVisit.Subject.Code.Contains(inQuery.SubjectCode))
                .WhereIf(inQuery.VisitPlanArray != null && inQuery.VisitPlanArray?.Length > 0, svExpression)
                //.WhereIf(!string.IsNullOrEmpty(challengeQuery.VisitPlanInfo), challengeQuery.VisitPlanInfo.Contains('.') ? t => t.SubjectVisit.InPlan == false : t => t.SubjectVisit.VisitNum == decimal.Parse(challengeQuery.VisitPlanInfo))
                .WhereIf(inQuery.IsUrgent != null, t => t.SubjectVisit.IsUrgent == inQuery.IsUrgent)
                .WhereIf(inQuery.IsOverTime != null && inQuery.IsOverTime == true, t => t.IsClosed ? t.ClosedTime > t.DeadlineTime : DateTime.Now > t.DeadlineTime)
                .WhereIf(inQuery.IsOverTime != null && inQuery.IsOverTime == false, t => t.IsClosed ? t.ClosedTime < t.DeadlineTime : DateTime.Now < t.DeadlineTime)
                .ProjectTo<QCCRCChallengeViewModel>(_mapper.ConfigurationProvider);

            var pageList = await query.ToPagedListAsync(inQuery, new string[] { nameof(QCCRCChallengeViewModel.IsUrgent) + " desc", nameof(QCCRCChallengeViewModel.CreateTime) });

            var config = await _trialRepository.Where(t => t.Id == inQuery.TrialId).ProjectTo<TrialSubjectAndSVConfig>(_mapper.ConfigurationProvider).FirstOrDefaultAsync().IfNullThrowException();
            return (pageList, config);
        }


        /// <summary>
        /// 获取下一个Qc信息
        /// </summary>
        /// <param name="inDto"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<QCVisitViewModel?> GetNextQCInfo(GetNextQCInfoInDto inDto)
        {
            var result = await GetQCVisitList(new QCVisitSearchDTO()
            {
                TrialId = inDto.TrialId,
                CurrentActionUserId = _userInfo.UserRoleId,
                VisitId = inDto.VisitId,
                PageIndex = 1,
                PageSize = 1,
            });

            return result.Data.CurrentPageData.Count > 0 ? result.Data.CurrentPageData[0] : null;
        }

        /// <summary>
        /// QC 访视列表
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<IResponseOutput<PageOutput<QCVisitViewModel>>> GetQCVisitList(QCVisitSearchDTO inQuery)
        {

            var svExpression = QCCommon.GetSubjectVisitFilter(inQuery.VisitPlanArray);
            var query = _subjectVisitRepository.Where(x => x.TrialId == inQuery.TrialId)
                        .WhereIf(inQuery.VisitId != null, t => t.Id == inQuery.VisitId)
                        .WhereIf(inQuery.CurrentActionUserId != null, t => t.CurrentActionUserId == inQuery.CurrentActionUserId)
                        .WhereIf(inQuery.ChallengeState != null, t => t.ChallengeState == inQuery.ChallengeState)
                        .WhereIf(inQuery.TrialSiteId != null, t => t.TrialSiteId == inQuery.TrialSiteId)
                        .WhereIf(inQuery.SubjectId != null, t => t.Subject.Id == inQuery.SubjectId)
                        .WhereIf(!string.IsNullOrEmpty(inQuery.SubjectInfo), t => /*t.Subject.FirstName.Contains(subjectInfo) || t.Subject.LastName.Contains(subjectInfo) ||*/ t.Subject.Code.Contains(inQuery.SubjectInfo))
                        .WhereIf(inQuery.VisitPlanArray != null && inQuery.VisitPlanArray?.Length > 0, svExpression)
                        //.WhereIf(!string.IsNullOrEmpty(visitSearchDTO.VisitPlanInfo), visitSearchDTO.VisitPlanInfo.Contains('.') ? t => t.VisitNum.ToString().Contains(".") : t => t.VisitNum == decimal.Parse(visitSearchDTO.VisitPlanInfo))
                        //.WhereIf(visitSearchDTO.AuditState != null, t => t.AuditState == visitSearchDTO.AuditState)
                        .WhereIf(inQuery.AuditStateArray != null && inQuery.AuditStateArray?.Length > 0, t => inQuery.AuditStateArray!.Contains(t.AuditState))

                        .WhereIf(inQuery.HandleUserId != null, t => t.PreliminaryAuditUserId == inQuery.HandleUserId || t.CurrentActionUserId == inQuery.HandleUserId
                        || t.ReviewAuditUserId == inQuery.HandleUserId
                        || t.QCChallengeList.Any(t => t.CreateUserId == inQuery.HandleUserId)
                        || t.QCChallengeDialogList.Any(t => t.CreateUserId == inQuery.HandleUserId))
                        .WhereIf(inQuery.IsUrgent != null, t => t.IsUrgent == inQuery.IsUrgent)
                        .Where(t => t.SubmitState != SubmitStateEnum.None)
                        .WhereIf(inQuery.BeginAuditTime != null, t => t.Trial.QCProcessEnum == TrialQCProcess.SingleAudit ? t.PreliminaryAuditTime >= inQuery.BeginAuditTime :
                        (t.Trial.QCProcessEnum == TrialQCProcess.DoubleAudit ? t.ReviewAuditTime >= inQuery.BeginAuditTime : true))

                         .WhereIf(inQuery.EndAuditTime != null, t => t.Trial.QCProcessEnum == TrialQCProcess.SingleAudit ? t.PreliminaryAuditTime <= inQuery.EndAuditTime :
                        (t.Trial.QCProcessEnum == TrialQCProcess.DoubleAudit ? t.ReviewAuditTime <= inQuery.EndAuditTime : true))

                        //.WhereIf(visitSearchDTO.SubmitState != null, t => t.SubmitState == visitSearchDTO.SubmitState)
                        //.WhereIf(visitSearchDTO.ChallengeState != null, t => t.ChallengeState == visitSearchDTO.ChallengeState)
                        .ProjectTo<QCVisitViewModel>(_mapper.ConfigurationProvider);

            var defalutSortArray = new string[] { nameof(QCVisitViewModel.IsUrgent) + " desc", nameof(QCVisitViewModel.SubjectCode), nameof(QCVisitViewModel.VisitNum) };
            //var defalutSortArray = new string[] { nameof(SubjectVisit.IsUrgent) + " desc", nameof(QCVisitViewModel.AuditState) +" asc" };
            var pageList = await query.ToPagedListAsync(inQuery, defalutSortArray);




            var config = await _trialRepository.Where(t => t.Id == inQuery.TrialId).ProjectTo<TrialSubjectAndSVConfig>(_mapper.ConfigurationProvider).FirstOrDefaultAsync().IfNullThrowException();

            return ResponseOutput.Ok(pageList, config);

        }

        #endregion


        #region 一致性核查 转发页面
        /// <summary>
        /// 获取一致性核查列表  CRC/PM 公用
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<(PageOutput<QCCheckWithModalityView>, TrialSubjectAndSVConfig)> GetConsistencyVerificationList(CheckQuery inQuery)
        {

            var svExpression = QCCommon.GetSubjectVisitFilter(inQuery.VisitPlanArray);

            var query = _subjectVisitRepository.Where(x => x.TrialId == inQuery.TrialId)
                        .Where(x => x.AuditState == AuditStateEnum.QCPassed) //一致性核查中的，或者还没一致性核查的                                                                                                                 
                        .WhereIf(inQuery.CheckState != null, t => t.CheckState == inQuery.CheckState)
                        .WhereIf(inQuery.TrialSiteId != null, t => t.TrialSiteId == inQuery.TrialSiteId)
                        .WhereIf(!string.IsNullOrEmpty(inQuery.SubjectInfo), t => t.Subject.Code.Contains(inQuery.SubjectInfo))
                        .WhereIf(inQuery.VisitPlanArray != null && inQuery.VisitPlanArray?.Length > 0, svExpression)
                        //.WhereIf(!string.IsNullOrEmpty(checkQuery.VisitPlanInfo), checkQuery.VisitPlanInfo.Contains('.') ? t => t.InPlan == false : t => t.VisitNum == decimal.Parse(checkQuery.VisitPlanInfo))
                        .WhereIf(_userInfo.UserTypeEnumInt == (int)UserTypeEnum.ClinicalResearchCoordinator || _userInfo.UserTypeEnumInt == (int)UserTypeEnum.CRA, t => t.TrialSite.CRCUserList.Any(t => t.UserId == _userInfo.UserRoleId))//CRC 过滤负责的site
                        .ProjectTo<QCCheckWithModalityView>(_mapper.ConfigurationProvider);

            var pageList = await query.ToPagedListAsync(inQuery);
            var config = await _trialRepository.Where(t => t.Id == inQuery.TrialId).ProjectTo<TrialSubjectAndSVConfig>(_mapper.ConfigurationProvider).FirstOrDefaultAsync().IfNullThrowException();
            return (pageList, config);
        }

        /// <summary>
        /// 一致性核查 聊天记录列表
        /// </summary>
        /// <param name="subjectVisitId"></param>
        /// <returns></returns>
        /// 

        [HttpGet("{subjectVisitId:guid}")]
        public async Task<CheckDialogDTO> GetCheckChallengeDialogList(Guid subjectVisitId)
        {

            var sv = (await _subjectVisitRepository.Where(t => t.Id == subjectVisitId)
                  .ProjectTo<CheckDialogDTO>(_mapper.ConfigurationProvider).FirstOrDefaultAsync()).IfNullThrowException();

            sv.DialogList.ForEach(t => t.IsCurrentUser = _userInfo.UserRoleId == t.CreateUserId);

            return sv;
        }

        /// <summary>
        /// 转发列表  查询一致性核查通过的，针对不一致性核查的 要维护CV状态
        /// </summary>
        /// <param name="inQuery"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<ForWardViewModel>> GetForwardList(ForwardQuery inQuery)
        {
            //var trialConfig = _trialReposioty.Select(t => new { TrialId = t.Id, t.QCProcessEnum, t.IsImageConsistencyVerification }).FirstOrDefault(t => t.TrialId == forwardQuery.TrialId);

            var svExpression = QCCommon.GetSubjectVisitFilter(inQuery.VisitPlanArray);

            var query = _subjectVisitRepository.Where(x => x.TrialId == inQuery.TrialId)
                        .Where(t => t.CheckState == CheckStateEnum.CVPassed)
                       .WhereIf(inQuery.ForwardState != null, t => t.ForwardState == inQuery.ForwardState)
                        .WhereIf(inQuery.TrialSiteId != null, t => t.TrialSiteId == inQuery.TrialSiteId)
                       .WhereIf(!string.IsNullOrEmpty(inQuery.SubjectInfo), t => t.Subject.Code.Contains(inQuery.SubjectInfo))
                        .WhereIf(inQuery.VisitPlanArray != null && inQuery.VisitPlanArray?.Length > 0, svExpression)
                       //.WhereIf(!string.IsNullOrEmpty(forwardQuery.VisitPlanInfo), forwardQuery.VisitPlanInfo.Contains('.') ? t => t.InPlan == false : t => t.VisitNum == decimal.Parse(forwardQuery.VisitPlanInfo))
                       //.WhereIf(_userInfo.UserTypeEnumInt == (int)UserType.ClinicalResearchCoordinator, t => t.TrialSite.CRCUserList.Any(t => t.UserId == _userInfo.Id))//CRC 过滤负责的site
                       .ProjectTo<ForWardViewModel>(_mapper.ConfigurationProvider);

            return await query.ToPagedListAsync(inQuery);

        }

        #endregion



        #region QC 具体质控页面 各种列表
        /// <summary>
        /// 获取某次访视 QA界面所有信息  单独每一项都有接口（往下看），这里是一个大接口，方便第一次获取完整的所有的数据
        /// </summary>
        /// <param name="subjectVisitId"></param>
        /// <param name="trialQCProcess">项目配置的针对访视检查是那种审核，0 不审，1 单审，2双审</param>
        /// <param name="currentQCType">当前 QC进入的是那种审核 1 单审，2复审</param>

        [HttpGet("{subjectVisitId:guid}/{trialQCProcess:int}/{currentQCType:int}")]
        public async Task<TrialVisitQADTO> GetVisitQCInfo(Guid subjectVisitId, [FromRoute] TrialQCProcess trialQCProcess, [FromRoute] CurrentQC currentQCType)
        {
            var sv = (await _subjectVisitRepository.FirstOrDefaultAsync(t => t.Id == subjectVisitId)).IfNullThrowException();

            var temp = await GetVisitQCStudyAndSeriesList(subjectVisitId);

            var qacheckList = await GetQCQuestionAnswerList(subjectVisitId, sv.TrialId, trialQCProcess, currentQCType);

            return new TrialVisitQADTO
            {
                QCQuestionAnswerList = qacheckList,

                IsHaveStudyClinicalData = await _clinicalDataTrialSetRepository.AnyAsync(x => x.IsConfirm && x.TrialId == sv.TrialId && x.ClinicalDataLevel == ClinicalLevel.Study),
                StudyList = temp.StudyList,
                ExistsManual = (await _IReadingImageTaskService.GetManualList(new GetManualListInDto() { TrialId = sv.TrialId })).Count() > 0,
                SeriesList = temp.SeriesList,
                RelationInfo = await GetVisitQCSubjectInfo(subjectVisitId),
                NoneDicomStudyList = await _noneDicomStudyRepository.Where(t => t.SubjectVisitId == subjectVisitId).ProjectTo<NoneDicomStudyView>(_mapper.ConfigurationProvider).ToListAsync(),
                SubjectClinicalData = await _subjectVisitRepository.Where(t => t.Id == subjectVisitId)
                .ProjectTo<SubjectClinicalDataDto>(_mapper.ConfigurationProvider, new { subjectVisitId = subjectVisitId, token = _userInfo.UserToken }).FirstOrDefaultAsync()
            };
        }

        /// <summary>
        /// 获取质控问题答案
        /// </summary>
        /// <param name="inDto"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<List<QCQuestionAnswer>> GetQCQuestionAnswer(GetQCQuestionAnswerInDto inDto)
        {
            var questionAnswerlist = await (from data in _trialQCQuestionRepository.Where(x => x.TrialId == inDto.TrialId && x.IsEnable)
                                            join answer in _trialQCQuestionAnswerRepository.Where(x => x.SubjectVisitId == inDto.SubjectVisitId && x.QCProcessEnum == inDto.QCProcessEnum && x.CurrentQCEnum == inDto.CurrentQCEnum).AsQueryable() on data.Id equals answer.TrialQCQuestionConfigureId into answertemp
                                            from leftanswer in answertemp.DefaultIfEmpty()
                                            select new QCQuestionAnswer()
                                            {
                                                Answer = leftanswer.Answer,
                                                ShowOrder = data.ShowOrder,
                                                QuestionName = data.QuestionName,
                                                Id = data.Id,
                                                IsRequired = data.IsRequired,
                                                ParentId = data.ParentId,
                                                ParentTriggerValue = data.ParentTriggerValue,
                                                Type = data.Type,
                                                TypeValue = data.TypeValue
                                            }).ToListAsync();

            var result = questionAnswerlist.Where(x => x.ParentId == null).ToList();
            result.ForEach(x =>
            {
                GetQuestionChild(x, questionAnswerlist);
            });

            return result;
        }


        private void GetQuestionChild(QCQuestionAnswer parent, List<QCQuestionAnswer> dataList)
        {
            parent.Childrens = dataList.Where(x => x.ParentId == parent.Id).ToList();

            if (parent.Childrens.Count != 0)
            {
                parent.Childrens.ForEach(x =>
                {
                    GetQuestionChild(x, dataList);
                });
            }


        }

        /// <summary>
        /// 获取某次访视 QC 问题核对答案 列表  初始化进去的时候是模板项，QC填写了就是对应的内容
        /// </summary>
        /// <param name="subjectVisitId"></param>
        /// <param name="trialId"></param>
        /// <param name="trialQCProcess">项目配置的针对访视检查是那种审核，0 不审，1 单审，2双审</param>
        /// <param name="currentQCType">当前 QC进入的是那种审核 1 单审，2复审</param>

        [HttpGet("{trialId:guid}/{subjectVisitId:guid}/{trialQCProcess:int}/{currentQCType:int}")]
        public async Task<List<QCQuestionAnswerItemDto>> GetQCQuestionAnswerList(Guid subjectVisitId, Guid trialId, [FromRoute] TrialQCProcess trialQCProcess, [FromRoute] CurrentQC currentQCType)
        {

            //https://github.com/dotnet/efcore/issues/26833  ef core 导航属性不支持右连接   

            // 直接用ProjectTo的方式   多做一次查询 代码简洁很多 特别是连表多，字段多的情况
            if (await _trialQCQuestionAnswerRepository.AnyAsync(t => t.SubjectVisitId == subjectVisitId && t.QCProcessEnum == trialQCProcess && t.CurrentQCEnum == currentQCType))
            {
                var list = await _trialQCQuestionAnswerRepository.Where(t => t.SubjectVisitId == subjectVisitId && t.QCProcessEnum == trialQCProcess && t.CurrentQCEnum == currentQCType).OrderBy(t => t.TrialQCQuestionConfigure.ShowOrder)
                       .ProjectTo<QCQuestionAnswerItemDto>(_mapper.ConfigurationProvider).ToListAsync();

                return list;
            }
            else
            {
                return await _trialQCQuestionRepository.Where(t => t.IsEnable == true && t.TrialId == trialId).OrderBy(t => t.ShowOrder).ProjectTo<QCQuestionAnswerItemDto>(_mapper.ConfigurationProvider).ToListAsync();
            }

        }


        /// <summary>
        /// 获次QC 历史质疑列表  不分页
        /// </summary>
        /// <param name="subjectVisitId"></param>
        /// <param name="trialQCProcess">项目配置的针对访视检查是那种审核，0 不审，1 单审，2双审</param>
        /// <param name="currentQCType">当前 QC进入的是那种审核 1 单审，2复审</param>

        [HttpGet("{subjectVisitId:guid}/{trialQCProcess:int}/{currentQCType:int}")]
        public async Task<List<QCChallengeWithUser>> GetHistoryChallengeList(Guid subjectVisitId, [FromRoute] TrialQCProcess trialQCProcess, [FromRoute] CurrentQC currentQCType)
        {
            var qaChallengeQuery = _qcChallengeRepository.Where(t => t.SubjectVisitId == subjectVisitId && t.QCProcessEnum == trialQCProcess)
                .WhereIf(currentQCType == CurrentQC.First, t => t.CurrentQCEnum == currentQCType)//复审的时候可以看到初审的质疑
                .ProjectTo<QCChallengeWithUser>(_mapper.ConfigurationProvider);
            var qaChallenges = await qaChallengeQuery.ToListAsync();

            return qaChallenges;
        }


        /// <summary>
        /// 获取访视下的受试者访视、受试者、site信息
        /// </summary>
        /// <param name="subjectVisitId"></param>
        /// <returns></returns>
        public async Task<QARelationInfo> GetVisitQCSubjectInfo(Guid subjectVisitId)
        {

            // automapper  Explicit expansion  ProjectTo 
            return await _subjectVisitRepository.Where(t => t.Id == subjectVisitId).ProjectTo<QARelationInfo>(_mapper.ConfigurationProvider).FirstOrDefaultAsync().IfNullThrowException();

        }



        /// <summary>
        ///   访视下的Study 和Series列表
        /// </summary>
        /// <param name="subjectVisitId"></param>
        /// <returns></returns>
        [HttpGet("{subjectVisitId:guid}")]
        public async Task<QAStudySeriesInfo> GetVisitQCStudyAndSeriesList(Guid subjectVisitId)
        {
            var studyList = await _dicomStudyRepository.Where(s => s.SubjectVisitId == subjectVisitId).IgnoreQueryFilters().ProjectTo<QAStudyInfoDTO>(_mapper.ConfigurationProvider).ToListAsync();
            var studyIds = studyList.Select(t => t.StudyId).ToList();

            var seriesList = await _dicomSeriesRepository.Where(t => studyIds.Contains(t.StudyId)).IgnoreQueryFilters().ProjectTo<QASeriesInfoDto>(_mapper.ConfigurationProvider)
                .OrderBy(t => t.StudyCode).ThenBy(t => t.SeriesNumber).ToListAsync();

            return new QAStudySeriesInfo { StudyList = studyList, SeriesList = seriesList };

        }

        /// <summary>
        /// 访视下的检查列表  
        /// </summary>
        /// <param name="subjectVisitId"></param>
        /// <returns></returns>
        [HttpGet("{subjectVisitId:guid}")]
        public async Task<IResponseOutput<List<QAStudyInfoDTO>>> GetSubjectVisitUploadedStudyList(Guid subjectVisitId)
        {

            var visit = await _subjectVisitRepository.Where(x => x.Id == subjectVisitId).FirstNotNullAsync();
            var list = await _dicomStudyRepository.Where(s => s.SubjectVisitId == subjectVisitId).IgnoreQueryFilters().ProjectTo<QAStudyInfoDTO>(_mapper.ConfigurationProvider).OrderBy(t => t.StudyCode).ToListAsync();


            var config = await _subjectVisitRepository.Where(t => t.Id == subjectVisitId).Select(t => t.Trial).ProjectTo<TrialSubjectAndSVConfig>(_mapper.ConfigurationProvider).FirstOrDefaultAsync().IfNullThrowException();

            config.IsHaveStudyClinicalData = await _clinicalDataTrialSetRepository.AnyAsync(x => x.IsConfirm && x.TrialId == visit.TrialId && x.ClinicalDataLevel == ClinicalLevel.Study);

            config.IsPacsConnectConfiged = await _subjectVisitRepository.Where(t => t.Id == subjectVisitId).AnyAsync(t => t.Trial.IsPACSConnect && t.Subject.TrialSite.TrialSiteDicomAEList.Any());

            return ResponseOutput.Ok (list, config);
        }


        /// <summary>
        /// 展开 某一QC质疑 下得 聊天记录 
        /// </summary>
        /// <param name="qaChallengeId"></param>
        /// <returns></returns>
        [HttpGet("{qaChallengeId:guid}")]
        public async Task<List<ChallengeAndDialog>> GetQCChallengeDialogList(Guid qaChallengeId)
        {

            var list = await _qcChallengeRepository.Where(t => t.Id == qaChallengeId)
                  .ProjectTo<ChallengeAndDialog>(_mapper.ConfigurationProvider, new { currentUserId = _userInfo.UserRoleId }).ToListAsync();

            //利用automapper 运行时映射
            //list.ForEach(t => t.IsCurrentUser = _userInfo.Id == t.CreateUserId);

            return list;
        }

        #endregion


        /// <summary>
        /// CRC/PM  看到某次访视下的所有质疑和聊天内容  包括初审和复审的 。
        /// </summary>
        /// <param name="subjectVisitId"></param>
        /// <param name="trialQCProcess"></param>
        /// <returns></returns>
        [HttpGet("{subjectVisitId:guid}/{trialQCProcess:int}")]
        public async Task<List<ChallengeAndDialog>> GetCRCVisitChallengeAndDialog(Guid subjectVisitId, [FromRoute] TrialQCProcess trialQCProcess)
        {

            var qaChallengeQuery = _qcChallengeRepository.Where(t => t.SubjectVisitId == subjectVisitId && t.QCProcessEnum == trialQCProcess)
                .ProjectTo<ChallengeAndDialog>(_mapper.ConfigurationProvider, new { currentUserId = _userInfo.UserRoleId });

            var list = await qaChallengeQuery.ToListAsync();

            //list.ForEach(t => t.DialogList.ToList().ForEach(u => u.IsCurrentUser = _userInfo.Id == u.CreateUserId));

            return list;
        }

        /// <summary>
        ///  质询发起人
        /// </summary>
        /// <param name="trialId"></param>
        /// <returns></returns>
        [HttpGet("{trialId:guid}")]
        public async Task<List<QCChanllengeCreatorDto>> GetQCChallengeCreatorList(Guid trialId)
        {

            return await _trialUserRoleRepository.Where(t => t.TrialId == trialId && t.UserRole.UserTypeEnum == UserTypeEnum.IQC).Select(t => new QCChanllengeCreatorDto()
            {
                CreatorRealName = t.UserRole.FullName,
                Creator = t.UserRole.IdentityUser.UserName,
                CreateUserId = t.UserId
            }).ToListAsync();

        }

        /// <summary>
        ///  QC  质疑页面  处理用户 下拉框
        /// </summary>
        /// <param name="trialId"></param>
        /// <returns></returns>
        [HttpGet("{trialId:guid}")]
        public async Task<List<ParticipantDTO>> GetQCParticipantList(Guid trialId)
        {
            return await _trialUserRoleRepository.Where(t => t.TrialId == trialId && (t.UserRole.UserTypeEnum == UserTypeEnum.IQC || t.UserRole.UserTypeEnum == UserTypeEnum.ClinicalResearchCoordinator)).Select(t => new ParticipantDTO()
            {
                HandleUserRealName = t.UserRole.FullName,
                HandleUser = t.UserRole.IdentityUser.UserName,
                HandleUserId = t.UserId
            }).ToListAsync();
        }



        /// <summary>
        /// 获取一致性核查文件上传记录
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<GetUserUploadFileDto>> GetConsistencyCheckFile(GetConsistencyCheckFileInDto inQuery, [FromServices] IRepository<InspectionFile> _inspectionFileRepository)
        {
            var query = _inspectionFileRepository.Where(t => t.TrialId == inQuery.TrialId)
             .ProjectTo<GetUserUploadFileDto>(_mapper.ConfigurationProvider);

            return await query.ToPagedListAsync(inQuery);
        }


        /// <summary>
        ///  添加计划外访视 下拉框 选择上一次访视
        /// </summary>
        /// <param name="subjectId"></param>
        /// <returns></returns>
        [HttpGet("{subjectId:guid}")]
        public async Task<List<SubjectVisitSelectItem>> GetSubjectVisitSelectList(Guid subjectId)
        {

            //var maxNum = await _visitTaskRepository.Where(t => t.SubjectId == subjectId && t.TaskState == TaskState.Effect && t.SignTime != null && t.TrialReadingCriterion.IsReadingTaskViewInOrder == ReadingOrder.InOrder).MaxAsync(x => (decimal?)x.VisitTaskNum)??0;

            var maxNum = await _subjectVisitRepository.Where(t => t.SubjectId == subjectId && t.SubmitState == SubmitStateEnum.Submitted).MaxAsync(x => (decimal?)x.VisitNum) ?? 0;

            return await _subjectVisitRepository.Where(t => t.SubjectId == subjectId && t.VisitNum >= maxNum).OrderBy(T => T.VisitNum).ProjectTo<SubjectVisitSelectItem>(_mapper.ConfigurationProvider).ToListAsync();
        }



        /// <summary>
        /// 上传界面 受试者 访视、site 基本信息
        /// </summary>
        /// <param name="subjectVisitId"></param>
        /// <returns></returns>
        [HttpGet("{subjectVisitId:guid}/{trialQCProcess:int}")]
        [Obsolete]
        public async Task<UploadViewInitDto> GetUploadInitInfo(Guid subjectVisitId)
        {
            return await _subjectVisitRepository.Where(t => t.Id == subjectVisitId).ProjectTo<UploadViewInitDto>(_mapper.ConfigurationProvider).FirstOrDefaultAsync().IfNullThrowException();
        }


    }
}
