﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

#nullable disable

namespace IRaCIS.Core.Infra.EFCore.Migrations
{
    /// <inheritdoc />
    public partial class CDISCCode : Migration
    {
        /// <inheritdoc />
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "SystemDocConfirmedUser");

            migrationBuilder.DropTable(
                name: "TrialDocConfirmedUser");

            migrationBuilder.AddColumn<string>(
                name: "CDISCCode",
                table: "ReadingTableQuestionTrial",
                type: "nvarchar(400)",
                maxLength: 400,
                nullable: false,
                defaultValue: "");

            migrationBuilder.AddColumn<string>(
                name: "CDISCCode",
                table: "ReadingQuestionTrial",
                type: "nvarchar(400)",
                maxLength: 400,
                nullable: false,
                defaultValue: "");
        }

        /// <inheritdoc />
        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropColumn(
                name: "CDISCCode",
                table: "ReadingTableQuestionTrial");

            migrationBuilder.DropColumn(
                name: "CDISCCode",
                table: "ReadingQuestionTrial");

            migrationBuilder.CreateTable(
                name: "SystemDocConfirmedUser",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    ConfirmUserId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    CreateUserId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    SystemDocumentId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    ConfirmTime = table.Column<DateTime>(type: "datetime2", nullable: true),
                    CreateTime = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DeleteUserId = table.Column<Guid>(type: "uniqueidentifier", nullable: true),
                    DeletedTime = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false),
                    SignFirstViewTime = table.Column<DateTime>(type: "datetime2", nullable: true),
                    SignText = table.Column<string>(type: "nvarchar(1000)", maxLength: 1000, nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_SystemDocConfirmedUser", x => x.Id);
                    table.ForeignKey(
                        name: "FK_SystemDocConfirmedUser_SystemDocument_SystemDocumentId",
                        column: x => x.SystemDocumentId,
                        principalTable: "SystemDocument",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_SystemDocConfirmedUser_User_ConfirmUserId",
                        column: x => x.ConfirmUserId,
                        principalTable: "User",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_SystemDocConfirmedUser_User_CreateUserId",
                        column: x => x.CreateUserId,
                        principalTable: "User",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                },
                comment: "后台 - 系统文档签署记录");

            migrationBuilder.CreateTable(
                name: "TrialDocConfirmedUser",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    ConfirmUserId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    CreateUserId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    TrialDocumentId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    ConfirmTime = table.Column<DateTime>(type: "datetime2", nullable: true),
                    CreateTime = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DeleteUserId = table.Column<Guid>(type: "uniqueidentifier", nullable: true),
                    DeletedTime = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false),
                    SignFirstViewTime = table.Column<DateTime>(type: "datetime2", nullable: true),
                    SignText = table.Column<string>(type: "nvarchar(1000)", maxLength: 1000, nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_TrialDocConfirmedUser", x => x.Id);
                    table.ForeignKey(
                        name: "FK_TrialDocConfirmedUser_TrialDocument_TrialDocumentId",
                        column: x => x.TrialDocumentId,
                        principalTable: "TrialDocument",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_TrialDocConfirmedUser_User_ConfirmUserId",
                        column: x => x.ConfirmUserId,
                        principalTable: "User",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_TrialDocConfirmedUser_User_CreateUserId",
                        column: x => x.CreateUserId,
                        principalTable: "User",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                },
                comment: "项目 - 项目文档签署记录");

            migrationBuilder.CreateIndex(
                name: "IX_SystemDocConfirmedUser_ConfirmUserId",
                table: "SystemDocConfirmedUser",
                column: "ConfirmUserId");

            migrationBuilder.CreateIndex(
                name: "IX_SystemDocConfirmedUser_CreateUserId",
                table: "SystemDocConfirmedUser",
                column: "CreateUserId");

            migrationBuilder.CreateIndex(
                name: "IX_SystemDocConfirmedUser_SystemDocumentId",
                table: "SystemDocConfirmedUser",
                column: "SystemDocumentId");

            migrationBuilder.CreateIndex(
                name: "IX_TrialDocConfirmedUser_ConfirmUserId",
                table: "TrialDocConfirmedUser",
                column: "ConfirmUserId");

            migrationBuilder.CreateIndex(
                name: "IX_TrialDocConfirmedUser_CreateUserId",
                table: "TrialDocConfirmedUser",
                column: "CreateUserId");

            migrationBuilder.CreateIndex(
                name: "IX_TrialDocConfirmedUser_TrialDocumentId",
                table: "TrialDocConfirmedUser",
                column: "TrialDocumentId");
        }
    }
}
